@@ -1,7 +1,7 @@
 /**************************************************************************
  *   definitions.h  --  This file is part of GNU nano.                    *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2017 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -41,10 +41,55 @@
 #include <limits.h>
 #endif
 
+/* Set a default value for PATH_MAX if there isn't one. */
+#ifndef PATH_MAX
+#define PATH_MAX  4096
+#endif
+
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 
+#include <dirent.h>
+#include <regex.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <sys/stat.h>
+
+/* Prefer wide ncurses over normal ncurses over curses. */
+#if defined(HAVE_NCURSESW_NCURSES_H)
+#include <ncursesw/ncurses.h>
+#elif defined(HAVE_NCURSES_H)
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif
+
+/* Native language support. */
+#ifdef ENABLE_NLS
+#ifdef HAVE_LIBINTL_H
+#include <libintl.h>
+#endif
+#define _(string)  gettext(string)
+#define P_(singular, plural, number)  ngettext(singular, plural, number)
+#else
+#define _(string)  (string)
+#define P_(singular, plural, number)  (number == 1 ? singular : plural)
+#endif
+/* For marking a string on which gettext() will be called later. */
+#define gettext_noop(string)  (string)
+#define N_(string)  gettext_noop(string)
+
+/* If we aren't using an ncurses with mouse support, then
+ * exclude the mouse routines, as they are useless then. */
+#ifndef NCURSES_MOUSE_VERSION
+#undef ENABLE_MOUSE
+#endif
+
+#if defined(ENABLE_WRAPPING) || defined(ENABLE_JUSTIFY)
+#define ENABLED_WRAPORJUSTIFY  1
+#endif
+
 /* Suppress warnings for __attribute__((warn_unused_result)). */
 #define IGNORE_CALL_RESULT(call)  do { if (call) {} } while(0)
 
@@ -56,66 +101,6 @@
 #define ISSET(flag)  ((FLAGS(flag) & FLAGMASK(flag)) != 0)
 #define TOGGLE(flag)  FLAGS(flag) ^= FLAGMASK(flag)
 
-/* In UTF-8 a character is at most six bytes long. */
-#ifdef ENABLE_UTF8
-#define MAXCHARLEN  6
-#else
-#define MAXCHARLEN  1
-#endif
-
-/* Set a default value for PATH_MAX if there isn't one. */
-#ifndef PATH_MAX
-#define PATH_MAX  4096
-#endif
-
-#ifdef USE_SLANG
-/* Slang support. */
-#include <slcurses.h>
-/* Slang curses emulation brain damage, part 3: Slang doesn't define the
- * curses equivalents of the Insert or Delete keys. */
-#define KEY_DC  SL_KEY_DELETE
-#define KEY_IC  SL_KEY_IC
-/* Ncurses support. */
-#elif defined(HAVE_NCURSESW_NCURSES_H)
-#include <ncursesw/ncurses.h>
-#elif defined(HAVE_NCURSES_H)
-#include <ncurses.h>
-#else
-/* Curses support. */
-#include <curses.h>
-#endif /* CURSES_H */
-
-#ifdef ENABLE_NLS
-/* Native language support. */
-#ifdef HAVE_LIBINTL_H
-#include <libintl.h>
-#endif
-#define _(string)  gettext(string)
-#define P_(singular, plural, number)  ngettext(singular, plural, number)
-#else
-#define _(string)  (string)
-#define P_(singular, plural, number)  (number == 1 ? singular : plural)
-#endif
-#define gettext_noop(string)  (string)
-#define N_(string)  gettext_noop(string)
-		/* Mark a string that will be sent to gettext() later. */
-
-#include <dirent.h>
-#include <regex.h>
-#include <signal.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-
-/* If we aren't using an ncurses with mouse support, exclude any
- * mouse routines, as they are useless then. */
-#ifndef NCURSES_MOUSE_VERSION
-#undef ENABLE_MOUSE
-#endif
-
-#if defined(ENABLE_WRAPPING) || defined(ENABLE_JUSTIFY)
-#define ENABLED_WRAPORJUSTIFY  1
-#endif
-
 #define BACKWARD  FALSE
 #define FORWARD  TRUE
 
@@ -126,9 +111,137 @@
 #define REPLACING  1
 #define INREGION   2
 
+/* In UTF-8 a character is at most six bytes long. */
+#ifdef ENABLE_UTF8
+#define MAXCHARLEN  6
+#else
+#define MAXCHARLEN  1
+#endif
+
+/* The default width of a tab in spaces. */
+#define WIDTH_OF_TAB  8
+
+/* The default number of columns from end of line where wrapping occurs. */
+#define COLUMNS_FROM_EOL  8
+
+/* The default comment character when a syntax does not specify any. */
+#define GENERAL_COMMENT_CHARACTER  "#"
+
+/* The maximum number of search/replace history strings saved. */
+#define MAX_SEARCH_HISTORY  100
+
+/* The largest size_t number that doesn't have the high bit set. */
+#define HIGHEST_POSITIVE  ((~(size_t)0) >> 1)
+
 #ifdef ENABLE_COLOR
 #define THE_DEFAULT  -1
 #define BAD_COLOR  -2
+
+/* Flags for indicating how a multiline regex pair apply to a line. */
+#define CNONE           (1<<1)
+		/* The start/end regexes don't cover this line at all. */
+#define CBEGINBEFORE    (1<<2)
+		/* The start regex matches on an earlier line, the end regex on this one. */
+#define CENDAFTER       (1<<3)
+		/* The start regex matches on this line, the end regex on a later one. */
+#define CWHOLELINE      (1<<4)
+		/* The start regex matches on an earlier line, the end regex on a later one. */
+#define CSTARTENDHERE   (1<<5)
+		/* Both the start and end regexes match within this line. */
+#define CWOULDBE        (1<<6)
+		/* An unpaired start match is on or before this line. */
+#endif
+
+/* Basic control codes. */
+#define ESC_CODE  0x1B
+#define DEL_CODE  0x7F
+
+/* Codes for "modified" Arrow keys, beyond KEY_MAX of ncurses. */
+#define CONTROL_LEFT    0x401
+#define CONTROL_RIGHT   0x402
+#define CONTROL_UP      0x403
+#define CONTROL_DOWN    0x404
+#define CONTROL_HOME    0x405
+#define CONTROL_END     0x406
+#define CONTROL_DELETE  0x40D
+#define SHIFT_CONTROL_LEFT    0x411
+#define SHIFT_CONTROL_RIGHT   0x412
+#define SHIFT_CONTROL_UP      0x413
+#define SHIFT_CONTROL_DOWN    0x414
+#define SHIFT_CONTROL_HOME    0x415
+#define SHIFT_CONTROL_END     0x416
+#define CONTROL_SHIFT_DELETE  0x41D
+#define ALT_LEFT      0x421
+#define ALT_RIGHT     0x422
+#define ALT_UP        0x423
+#define ALT_DOWN      0x424
+#define ALT_PAGEUP    0x427
+#define ALT_PAGEDOWN  0x428
+#define ALT_INSERT    0x42C
+#define ALT_DELETE    0x42D
+#define SHIFT_ALT_LEFT   0x431
+#define SHIFT_ALT_RIGHT  0x432
+#define SHIFT_ALT_UP     0x433
+#define SHIFT_ALT_DOWN   0x434
+//#define SHIFT_LEFT 0x451
+//#define SHIFT_RIGHT 0x452
+#define SHIFT_UP        0x453
+#define SHIFT_DOWN      0x454
+#define SHIFT_HOME      0x455
+#define SHIFT_END       0x456
+#define SHIFT_PAGEUP    0x457
+#define SHIFT_PAGEDOWN  0x458
+#define SHIFT_DELETE    0x45D
+#define SHIFT_TAB       0x45F
+
+/* A special keycode for when <Tab> is pressed while the mark is on. */
+#define INDENT_KEY  0x4F1
+
+/* A special keycode to signal the beginning and end of a bracketed paste. */
+#define BRACKETED_PASTE_MARKER  0x4FB
+
+/* A special keycode for when a key produces an unknown escape sequence. */
+#define FOREIGN_SEQUENCE  0x4FC
+
+/* A special keycode for plugging into the input stream after a suspension. */
+#define KEY_FLUSH  KEY_F0
+
+#ifndef NANO_TINY
+/* A special keycode for when we get a SIGWINCH (a window resize). */
+#define KEY_WINCH  -2
+
+/* Some extra flags for the undo function. */
+#define WAS_BACKSPACE_AT_EOF  (1<<1)
+#define WAS_WHOLE_LINE        (1<<2)
+#define INCLUDED_LAST_LINE    (1<<3)
+#define MARK_WAS_SET          (1<<4)
+#define CURSOR_WAS_AT_HEAD    (1<<5)
+#endif /* !NANO_TINY */
+
+/* Identifiers for the different menus. */
+#define MMAIN          (1<<0)
+#define MWHEREIS       (1<<1)
+#define MREPLACE       (1<<2)
+#define MREPLACEWITH   (1<<3)
+#define MGOTOLINE      (1<<4)
+#define MWRITEFILE     (1<<5)
+#define MINSERTFILE    (1<<6)
+#define MEXECUTE       (1<<7)
+#define MHELP          (1<<8)
+#define MSPELL         (1<<9)
+#define MBROWSER      (1<<10)
+#define MWHEREISFILE  (1<<11)
+#define MGOTODIR      (1<<12)
+#define MYESNO        (1<<13)
+#define MLINTER       (1<<14)
+#define MFINDINHELP   (1<<15)
+/* This is an abbreviation for all menus except Help and Browser and YesNo. */
+#define MMOST  (MMAIN|MWHEREIS|MREPLACE|MREPLACEWITH|MGOTOLINE|MWRITEFILE|MINSERTFILE|\
+                MEXECUTE|MWHEREISFILE|MGOTODIR|MFINDINHELP|MSPELL|MLINTER)
+#ifndef NANO_TINY
+#define MSOME  MMOST|MBROWSER
+#else
+#define MSOME  MMAIN|MBROWSER
 #endif
 
 /* Enumeration types. */
@@ -137,17 +250,13 @@ typedef enum {
 } format_type;
 
 typedef enum {
-	VACUUM, HUSH, NOTICE, MILD, ALERT
+	VACUUM, HUSH, REMARK, INFO, NOTICE, AHEM, MILD, ALERT
 } message_type;
 
 typedef enum {
 	OVERWRITE, APPEND, PREPEND
 } kind_of_writing_type;
 
-typedef enum {
-	SOFTMARK, HARDMARK
-} mark_type;
-
 typedef enum {
 	CENTERING, FLOWING, STATIONARY
 } update_type;
@@ -166,6 +275,78 @@ typedef enum {
 	COUPLE_BEGIN, COUPLE_END, OTHER
 } undo_type;
 
+/* The elements of the interface that can be colored differently. */
+enum {
+	TITLE_BAR = 0,
+	LINE_NUMBER,
+	GUIDE_STRIPE,
+	SCROLL_BAR,
+	SELECTED_TEXT,
+	PROMPT_BAR,
+	STATUS_BAR,
+	ERROR_MESSAGE,
+	KEY_COMBO,
+	FUNCTION_TAG,
+	NUMBER_OF_ELEMENTS
+};
+
+/* Enumeration used in the flags array.  See the definition of FLAGMASK. */
+enum {
+	DONTUSE = 0,
+	CASE_SENSITIVE,
+	CONSTANT_SHOW,
+	NO_HELP,
+	SUSPENDABLE,
+	NO_WRAP,
+	AUTOINDENT,
+	VIEW_MODE,
+	USE_MOUSE,
+	USE_REGEXP,
+	SAVE_ON_EXIT,
+	CUT_FROM_CURSOR,
+	BACKWARDS_SEARCH,
+	MULTIBUFFER,
+	SMOOTH_SCROLL,
+	REBIND_DELETE,
+	RAW_SEQUENCES,
+	NO_CONVERT,
+	MAKE_BACKUP,
+	INSECURE_BACKUP,
+	NO_SYNTAX,
+	PRESERVE,
+	HISTORYLOG,
+	RESTRICTED,
+	SMART_HOME,
+	WHITESPACE_DISPLAY,
+	MORE_SPACE,
+	TABS_TO_SPACES,
+	QUICK_BLANK,
+	WORD_BOUNDS,
+	NO_NEWLINES,
+	BOLD_TEXT,
+	SOFTWRAP,
+	POSITIONLOG,
+	LOCKING,
+	NOREAD_MODE,
+	MAKE_IT_UNIX,
+	TRIM_BLANKS,
+	SHOW_CURSOR,
+	LINE_NUMBERS,
+	NO_PAUSES,
+	AT_BLANKS,
+	AFTER_ENDS,
+	LET_THEM_ZAP,
+	BREAK_LONG_LINES,
+	JUMPY_SCROLLING,
+	EMPTY_LINE,
+	INDICATOR,
+	BOOKSTYLE,
+	STATEFLAGS,
+	USE_MAGIC,
+	MINIBAR,
+	MARK_MATCH
+};
+
 /* Structure types. */
 #ifdef ENABLE_COLOR
 typedef struct colortype {
@@ -212,7 +393,7 @@ typedef struct syntaxtype {
 		/* File where the syntax is defined, or NULL if not an included file. */
 	size_t lineno;
 		/* The line number where the 'syntax' command was found. */
-	struct augmentstruct *augmentations;
+	augmentstruct *augmentations;
 		/* List of extendsyntax commands to apply when loaded. */
 	regexlisttype *extensions;
 		/* The list of extensions that this syntax applies to. */
@@ -252,20 +433,6 @@ typedef struct lintstruct {
 	struct lintstruct *prev;
 		/* Previous error. */
 } lintstruct;
-
-/* Flags that indicate how a multiline regex applies to a line. */
-#define CNONE           (1<<1)
-		/* Yay, regex doesn't apply to this line at all! */
-#define CBEGINBEFORE    (1<<2)
-		/* Regex starts on an earlier line, ends on this one. */
-#define CENDAFTER       (1<<3)
-		/* Regex starts on this line and ends on a later one. */
-#define CWHOLELINE      (1<<4)
-		/* Whole line engulfed by the regex, start < me, end > me. */
-#define CSTARTENDHERE   (1<<5)
-		/* Regex starts and ends within this line. */
-#define CWOULDBE        (1<<6)
-		/* An unpaired start match on or before this line. */
 #endif /* ENABLE_COLOR */
 
 /* More structure types. */
@@ -378,8 +545,8 @@ typedef struct openfilestruct {
 		/* The line in the file where the mark is set; NULL if not set. */
 	size_t mark_x;
 		/* The mark's x position in the above line. */
-	mark_type kind_of_mark;
-		/* Whether it is a soft (with Shift) or a hard mark. */
+	bool softmark;
+		/* Whether a marked region was made by holding Shift. */
 	format_type fmt;
 		/* The file's format -- Unix or DOS or Mac. */
 	char *lock_filename;
@@ -466,185 +633,3 @@ typedef struct completion_word {
 	struct completion_word *next;
 } completion_word;
 #endif
-
-/* The elements of the interface that can be colored differently. */
-enum
-{
-	TITLE_BAR = 0,
-	LINE_NUMBER,
-	GUIDE_STRIPE,
-	SCROLL_BAR,
-	SELECTED_TEXT,
-	STATUS_BAR,
-	ERROR_MESSAGE,
-	KEY_COMBO,
-	FUNCTION_TAG,
-	NUMBER_OF_ELEMENTS
-};
-
-/* Enumeration used in the flags array.  See the definition of FLAGMASK. */
-enum
-{
-	DONTUSE,
-	CASE_SENSITIVE,
-	CONSTANT_SHOW,
-	NO_HELP,
-	SUSPENDABLE,
-	NO_WRAP,
-	AUTOINDENT,
-	VIEW_MODE,
-	USE_MOUSE,
-	USE_REGEXP,
-	SAVE_ON_EXIT,
-	CUT_FROM_CURSOR,
-	BACKWARDS_SEARCH,
-	MULTIBUFFER,
-	SMOOTH_SCROLL,
-	REBIND_DELETE,
-	RAW_SEQUENCES,
-	NO_CONVERT,
-	MAKE_BACKUP,
-	INSECURE_BACKUP,
-	NO_SYNTAX,
-	PRESERVE,
-	HISTORYLOG,
-	RESTRICTED,
-	SMART_HOME,
-	WHITESPACE_DISPLAY,
-	MORE_SPACE,
-	TABS_TO_SPACES,
-	QUICK_BLANK,
-	WORD_BOUNDS,
-	NO_NEWLINES,
-	BOLD_TEXT,
-	SOFTWRAP,
-	POSITIONLOG,
-	LOCKING,
-	NOREAD_MODE,
-	MAKE_IT_UNIX,
-	TRIM_BLANKS,
-	SHOW_CURSOR,
-	LINE_NUMBERS,
-	NO_PAUSES,
-	AT_BLANKS,
-	AFTER_ENDS,
-	LET_THEM_ZAP,
-	BREAK_LONG_LINES,
-	JUMPY_SCROLLING,
-	EMPTY_LINE,
-	INDICATOR,
-	BOOKSTYLE,
-	STATEFLAGS,
-	USE_MAGIC
-};
-
-/* Flags for the menus in which a given function should be present. */
-#define MMAIN           (1<<0)
-#define MWHEREIS        (1<<1)
-#define MREPLACE        (1<<2)
-#define MREPLACEWITH    (1<<3)
-#define MGOTOLINE       (1<<4)
-#define MWRITEFILE      (1<<5)
-#define MINSERTFILE     (1<<6)
-#define MEXECUTE        (1<<7)
-#define MHELP           (1<<8)
-#define MSPELL          (1<<9)
-#define MBROWSER        (1<<10)
-#define MWHEREISFILE    (1<<11)
-#define MGOTODIR        (1<<12)
-#define MYESNO          (1<<13)
-#define MLINTER         (1<<14)
-#define MFINDINHELP     (1<<15)
-/* This is an abbreviation for all menus except Help and Browser and YesNo. */
-#define MMOST  (MMAIN|MWHEREIS|MREPLACE|MREPLACEWITH|MGOTOLINE|MWRITEFILE|MINSERTFILE|\
-                MEXECUTE|MWHEREISFILE|MGOTODIR|MFINDINHELP|MSPELL|MLINTER)
-#ifndef NANO_TINY
-#define MSOME  MMOST|MBROWSER
-#else
-#define MSOME  MMAIN|MBROWSER
-#endif
-
-/* Basic control codes. */
-#define ESC_CODE  0x1B
-#define DEL_CODE  0x7F
-
-/* Codes for "modified" Arrow keys, beyond KEY_MAX of ncurses. */
-#define CONTROL_LEFT    0x401
-#define CONTROL_RIGHT   0x402
-#define CONTROL_UP      0x403
-#define CONTROL_DOWN    0x404
-#define CONTROL_HOME    0x405
-#define CONTROL_END     0x406
-#define CONTROL_DELETE  0x40D
-#define SHIFT_CONTROL_LEFT    0x411
-#define SHIFT_CONTROL_RIGHT   0x412
-#define SHIFT_CONTROL_UP      0x413
-#define SHIFT_CONTROL_DOWN    0x414
-#define SHIFT_CONTROL_HOME    0x415
-#define SHIFT_CONTROL_END     0x416
-#define CONTROL_SHIFT_DELETE  0x41D
-#define ALT_LEFT      0x421
-#define ALT_RIGHT     0x422
-#define ALT_UP        0x423
-#define ALT_DOWN      0x424
-#define ALT_PAGEUP    0x427
-#define ALT_PAGEDOWN  0x428
-#define ALT_INSERT    0x42C
-#define ALT_DELETE    0x42D
-#define SHIFT_ALT_LEFT   0x431
-#define SHIFT_ALT_RIGHT  0x432
-#define SHIFT_ALT_UP     0x433
-#define SHIFT_ALT_DOWN   0x434
-//#define SHIFT_LEFT 0x451
-//#define SHIFT_RIGHT 0x452
-#define SHIFT_UP        0x453
-#define SHIFT_DOWN      0x454
-#define SHIFT_HOME      0x455
-#define SHIFT_END       0x456
-#define SHIFT_PAGEUP    0x457
-#define SHIFT_PAGEDOWN  0x458
-#define SHIFT_DELETE    0x45D
-#define SHIFT_TAB       0x45F
-
-/* A special keycode for when <Tab> is pressed while the mark is on. */
-#define INDENT_KEY  0x4F1
-
-/* A special keycode to signal the beginning and end of a bracketed paste. */
-#define BRACKETED_PASTE_MARKER  0x4FB
-
-/* A special keycode for when a key produces an unknown escape sequence. */
-#define FOREIGN_SEQUENCE  0x4FC
-
-#ifdef USE_SLANG
-#define KEY_FLUSH  0xFF  /* Clipped error code. */
-#else
-#define KEY_FLUSH  KEY_F0  /* Nonexistent function key. */
-#endif
-
-#ifndef NANO_TINY
-/* An imaginary key for when we get a SIGWINCH (window resize). */
-#define KEY_WINCH  -2
-
-/* Some extra flags for the undo function. */
-#define WAS_BACKSPACE_AT_EOF  (1<<1)
-#define WAS_WHOLE_LINE        (1<<2)
-#define INCLUDED_LAST_LINE    (1<<3)
-#define MARK_WAS_SET          (1<<4)
-#define CURSOR_WAS_AT_HEAD    (1<<5)
-#endif /* !NANO_TINY */
-
-/* The default number of columns from end of line where wrapping occurs. */
-#define COLUMNS_FROM_EOL  8
-
-/* The default width of a tab in spaces. */
-#define WIDTH_OF_TAB  8
-
-/* The default comment character when a syntax does not specify any. */
-#define GENERAL_COMMENT_CHARACTER  "#"
-
-/* The maximum number of search/replace history strings saved, not
- * counting the blank lines at their ends. */
-#define MAX_SEARCH_HISTORY  100
-
-/* The largest size_t number that doesn't have the high bit set. */
-#define HIGHEST_POSITIVE  ((~(size_t)0) >> 1)
